/*
 * sample.c
 *
 * Image library
 *
 * Description:
 *	Portable routines to manipulate raster images.
 *
 * History:
 *	12-Apr-99	Craig Knudsen	cknudsen@radix.net
 *			Fixed to use real copyright ASCII code
 *			rather than escape.
 *	17-May-98	Craig Knudsen	cknudsen@radix.net
 *			Updated date to 1998
 *			Added call to ITextDimensions to center text.
 *	20-May-96	Craig Knudsen	cknudsen@radix.net
 *			Created
 *
 ****************************************************************************/

#include <stdio.h>
#include <stdlib.h>
#include <ctype.h>

#include <Ilib.h>

#define DEFAULT_FONTNAME_1	"helvR24"
#define DEFAULT_FONT_1		helvR24_font
#define DEFAULT_FONTNAME_2	"helvR08"
#define DEFAULT_FONT_2		helvR08_font

#include "helvR24.h"
#include "helvR08.h"


int main ( argc, argv )
int argc;
char *argv[];
{
  IImage image;
  IFont font1, font2;
  char *fontname1 = NULL, *fontname2 = NULL;
  int loop;
  unsigned int width = 400, height = 150, text_width = 0, text_height = 0,
    font_height = 0, dummy;
  int x, y;
  char *outfile = "out.gif";
  char *sample_text = NULL;
  char *copyright = "@Copyright 1999 Craig Knudsen";
#ifdef HAVE_GIFLIB
  int output_format = IFORMAT_GIF;
#else
  int output_format = IFORMAT_PPM;
#endif
  char *infile = NULL;
  FILE *fp;
  IGC gc;
  IError ret;
  IColor topshadow, bottomshadow, topfont, bottomfont, background;

  for ( loop = 1; loop < argc; loop++ ) {
    if ( strcmp ( argv[loop], "-w" ) == 0 ||
      strcmp ( argv[loop], "-width" ) == 0 ) {
      if ( ++loop >= argc ) {
        fprintf ( stderr, "-width requires an argument\n" );
        exit ( 1 );
      }
      width = atoi ( argv[loop] );
    }
    else if ( strcmp ( argv[loop], "-h" ) == 0 ||
      strcmp ( argv[loop], "-height" ) == 0 ) {
      if ( ++loop >= argc ) {
        fprintf ( stderr, "-height requires an argument\n" );
        exit ( 1 );
      }
      height = atoi ( argv[loop] );
    }
    else if ( strcmp ( argv[loop], "-font" ) == 0 ) {
      if ( ++loop >= argc ) {
        fprintf ( stderr, "-font requires an argument\n" );
        exit ( 1 );
      }
      fontname1 = argv[loop];
    }
    else if ( strcmp ( argv[loop], "-t" ) == 0 ||
      strcmp ( argv[loop], "-text" ) == 0 ) {
      if ( ++loop >= argc ) {
        fprintf ( stderr, "-text requires an argument\n" );
        exit ( 1 );
      }
      sample_text = argv[loop];
    }
    else if ( strcmp ( argv[loop], "-o" ) == 0 ||
      strcmp ( argv[loop], "-out" ) == 0 ) {
      if ( ++loop >= argc ) {
        fprintf ( stderr, "-out requires an argument\n" );
        exit ( 1 );
      }
      outfile = argv[loop];
    }
    else if ( strcmp ( argv[loop], "-i" ) == 0 ||
      strcmp ( argv[loop], "-infile" ) == 0 ) {
      if ( ++loop >= argc ) {
        fprintf ( stderr, "-infile requires an argument\n" );
        exit ( 1 );
      }
      infile = argv[loop];
    }
    else if ( strcmp ( argv[loop], "-format" ) == 0 ) {
      if ( ++loop >= argc ) {
        fprintf ( stderr, "-format requires an argument\n" );
        exit ( 1 );
      }
      if ( strcmp ( argv[loop], "xpm" ) == 0 )
        output_format = IFORMAT_XPM;
      else if ( strcmp ( argv[loop], "ppm" ) == 0 )
        output_format = IFORMAT_PPM;
#ifdef HAVE_GIFLIB
      else if ( strcmp ( argv[loop], "gif" ) == 0 )
        output_format = IFORMAT_GIF;
#endif
      else {
        fprintf ( stderr, "Unsupported output format: %s\n", argv[loop] );
        exit ( 1 );
      }
    }
  }

  if ( sample_text == NULL ) {
    sample_text = (char *) malloc ( 100 );
    sprintf ( sample_text, "Ilib v%s (%s)\n%s", ILIB_VERSION, ILIB_VERSION_DATE,
      ILIB_URL );
  }
  
  if ( infile ) {
    fp = fopen ( infile, "r" );
    if ( ! fp ) {
      perror ( "Error opening input file:" );
      exit ( 1 );
    }
    if ( ( ret = IReadImageFile ( fp, IFORMAT_PPM, IOPTION_NONE, &image ) ) ) {
      fprintf ( stderr, "Error reading image: %d\n", ret );
      exit ( 1 );
    }
    fclose ( fp );
  }
  else {
    image = ICreateImage ( width, height, IOPTION_NONE );
  }

  gc = ICreateGC ();
  topshadow = IAllocColor ( 100, 100, 100 );
  bottomshadow = IAllocColor ( 200, 200, 200 );
  topfont = IAllocColor ( 255, 255, 255 );
  bottomfont = IAllocColor ( 200, 50, 50 );
  background = IAllocColor ( 150, 150, 150 );

  /* draw top shadow rectangle */
  ISetForeground ( gc, topshadow );
  IFillRectangle ( image, gc, 0, 0, width, height );

  /* draw bottom shadow rectangle */
  ISetForeground ( gc, topshadow );
  IFillRectangle ( image, gc, 2, 2, width - 2, height - 2 );

  /* draw background rectangle */
  ISetForeground ( gc, background );
  IFillRectangle ( image, gc, 2, 2, width - 4, height - 4 );

  /* Now the fun part: draw some text */
  if ( fontname1 ) {
    if ( ( ret = ILoadFontFromFile ( fontname1, fontname1, &font1 ) ) ) {
      fprintf ( stderr, "Error (%s) loading font: %s\n",
        IErrorString ( ret ), fontname1 );
      exit ( 1 );
    }
  }
  else {
    fontname1 = DEFAULT_FONTNAME_1;
    if ( ( ret = ILoadFontFromData ( fontname1, DEFAULT_FONT_1, &font1 ) ) ) {
      fprintf ( stderr, "Error (%s) loading font: %s\n", 
        IErrorString ( ret ), fontname1 );
      exit ( 1 );
    }
  }
  fontname2 = DEFAULT_FONTNAME_2;
  if ( ( ret = ILoadFontFromData ( fontname2, DEFAULT_FONT_2, &font2 ) ) ) {
    fprintf ( stderr, "Error (%s) loading font: %s\n", IErrorString ( ret ),
      fontname2 );
    exit ( 1 );
  }

  ISetFont ( gc, font1 );
  ISetForeground ( gc, bottomfont );
  ret = ITextDimensions ( gc, font1, sample_text, strlen ( sample_text ),
    &text_width, &text_height );
  ret = ITextDimensions ( gc, font1, "X", 1, &dummy, &font_height );
  x = ( (int)width - (int)text_width ) / 2;
  y = ( ( (int)height - (int)text_height ) / 2 ) + (int)font_height;
  
  IDrawString ( image, gc, x, y, sample_text, strlen ( sample_text ) );
  ISetForeground ( gc, topfont );
  IDrawString ( image, gc, x - 1, y - 1, sample_text, strlen ( sample_text ) );

  y += text_height;

  /*ISetAntiAliasedFont ( gc, font2 );*/
  ISetFont ( gc, font2 );
  ret = ITextDimensions ( gc, font2, sample_text, strlen ( sample_text ),
    &text_width, &text_height );
  ISetForeground ( gc, bottomfont );
  IDrawString ( image, gc, x, y, copyright, strlen ( copyright ) );
  ISetForeground ( gc, topfont );
  IDrawString ( image, gc, x - 1, y - 1, copyright, strlen ( copyright ) );

  if ( outfile ) {
    fp = fopen ( outfile, "w" );
    if ( ! fp ) {
      perror ( "Cannot open output file: " );
      exit ( 1 );
    }
  }
  else
    fp = stdout;

  IWriteImageFile ( fp, image, output_format, IOPTION_INTERLACED );

  fclose ( fp );

  IFreeFont ( font1 );
  IFreeFont ( font2 );

  return ( 0 );
}


